@@ -38,7 +38,7 @@ impl NapiConst {
       }
       #[allow(non_snake_case)]
       #[allow(clippy::all)]
-      #[cfg(not(test))]
+      #[cfg(all(not(test), not(feature = "noop")))]
       #[napi::bindgen_prelude::ctor]
       fn #register_name() {
         napi::bindgen_prelude::register_module_export(#js_mod_ident, #js_name_lit, #cb_name);
