@@ -59,9 +59,14 @@ impl ToTypeDef for NapiFn {
     }
 
     let def = format!(
-      r#"{prefix} {name}({args}){ret}"#,
+      r#"{prefix} {name}{generic}({args}){ret}"#,
       prefix = self.gen_ts_func_prefix(),
       name = &self.js_name,
+      generic = &self
+        .ts_generic_types
+        .as_ref()
+        .map(|g| format!("<{}>", g))
+        .unwrap_or_else(|| "".to_string()),
       args = self
         .ts_args_type
         .clone()
@@ -92,7 +97,7 @@ fn gen_callback_type(callback: &CallbackArg) -> String {
       .iter()
       .enumerate()
       .map(|(i, arg)| {
-        let (ts_type, is_optional) = ty_to_ts_type(arg, false);
+        let (ts_type, is_optional) = ty_to_ts_type(arg, false, false);
         FnArg {
           arg: format!("arg{}", i),
           ts_type,
@@ -101,7 +106,7 @@ fn gen_callback_type(callback: &CallbackArg) -> String {
       })
       .collect::<FnArgList>(),
     ret = match &callback.ret {
-      Some(ty) => ty_to_ts_type(ty, true).0,
+      Some(ty) => ty_to_ts_type(ty, true, false).0,
       None => "void".to_owned(),
     }
   )
@@ -116,7 +121,8 @@ impl NapiFn {
         .iter()
         .filter_map(|arg| match arg {
           crate::NapiFnArgKind::PatType(path) => {
-            if path.ty.to_token_stream().to_string() == "Env" {
+            let ty_string = path.ty.to_token_stream().to_string();
+            if ty_string == "Env" || ty_string.replace(' ', "").starts_with("Reference<") {
               return None;
             }
             let mut path = path.clone();
@@ -125,7 +131,7 @@ impl NapiFn {
               i.mutability = None;
             }
             let arg = path.pat.to_token_stream().to_string().to_case(Case::Camel);
-            let (ts_type, is_optional) = ty_to_ts_type(&path.ty, false);
+            let (ts_type, is_optional) = ty_to_ts_type(&path.ty, false, false);
 
             Some(FnArg {
               arg,
@@ -175,7 +181,7 @@ impl NapiFn {
         .unwrap_or_else(|| "".to_owned()),
       _ => {
         let ret = if let Some(ret) = &self.ret {
-          let (ts_type, _) = ty_to_ts_type(ret, true);
+          let (ts_type, _) = ty_to_ts_type(ret, true, false);
           if ts_type == "undefined" {
             "void".to_owned()
           } else if ts_type == "Self" {
